@@ -1,6 +1,8 @@
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# Copyright (C) 2023 Fortra. All rights reserved.
+# Copyright Fortra, LLC and its affiliated companies 
+#
+# All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -36,6 +38,7 @@ class NTLMRelayxConfig:
         self.mode = None
         self.redirecthost = None
         self.outputFile = None
+        self.dumpHashes = False
         self.attacks = None
         self.lootdir = None
         self.randomtargets = False
@@ -43,6 +46,7 @@ class NTLMRelayxConfig:
         self.ipv6 = False
         self.remove_mic = False
         self.disableMulti = False
+        self.keepRelaying = False
 
         self.command = None
 
@@ -57,6 +61,7 @@ class NTLMRelayxConfig:
         self.interactive = False
         self.enumLocalAdmins = False
         self.SMBServerChallenge = None
+        self.rpc_attack = None
 
         # RPC options
         self.rpc_mode = None
@@ -100,9 +105,20 @@ class NTLMRelayxConfig:
         self.ShadowCredentialsExportType = None
         self.ShadowCredentialsOutfilePath = None
 
+        # SCCM attacks options
+        self.isSCCMPoliciesAttack = False
+        self.SCCMPoliciesClientname = None
+        self.SCCMPoliciesSleep = None
+        self.isSCCMDPAttack = False
+        self.SCCMDPExtensions = None
+        self.SCCMDPFiles = None
+
     def setSMBChallenge(self, value):
         self.SMBServerChallenge = value
 
+    def setSMBRPCAttack(self, value):
+        self.rpc_attack = value
+
     def setSMB2Support(self, value):
         self.smb2support = value
 
@@ -122,6 +138,9 @@ class NTLMRelayxConfig:
     def setOutputFile(self, outputFile):
         self.outputFile = outputFile
 
+    def setdumpHashes(self, dumpHashes):
+        self.dumpHashes = dumpHashes
+
     def setTargets(self, target):
         self.target = target
 
@@ -134,9 +153,15 @@ class NTLMRelayxConfig:
     def setEnumLocalAdmins(self, enumLocalAdmins):
         self.enumLocalAdmins = enumLocalAdmins
 
+    def setAddComputerSMB(self, addComputerSMB):
+        self.addComputerSMB = addComputerSMB
+
     def setDisableMulti(self, disableMulti):
         self.disableMulti = disableMulti
 
+    def setKeepRelaying(self, keepRelaying):
+        self.keepRelaying = keepRelaying
+
     def setEncoding(self, encoding):
         self.encoding = encoding
 
@@ -182,7 +207,7 @@ class NTLMRelayxConfig:
     def setMSSQLOptions(self, queries):
         self.queries = queries
 
-    def setRPCOptions(self, rpc_mode, rpc_use_smb, auth_smb, hashes_smb, rpc_smb_port):
+    def setRPCOptions(self, rpc_mode, rpc_use_smb, auth_smb, hashes_smb, rpc_smb_port, icpr_ca_name):
         self.rpc_mode = rpc_mode
         self.rpc_use_smb = rpc_use_smb
         self.smbdomain, self.smbuser, self.smbpass = parse_credentials(auth_smb)
@@ -194,6 +219,7 @@ class NTLMRelayxConfig:
             self.smbnthash = ''
 
         self.rpc_smb_port = rpc_smb_port
+        self.icpr_ca_name = icpr_ca_name
 
     def setInteractive(self, interactive):
         self.interactive = interactive
@@ -234,6 +260,20 @@ class NTLMRelayxConfig:
         self.ShadowCredentialsPFXPassword = ShadowCredentialsPFXPassword
         self.ShadowCredentialsExportType = ShadowCredentialsExportType
         self.ShadowCredentialsOutfilePath = ShadowCredentialsOutfilePath
+    
+    def setIsSCCMPoliciesAttack(self, isSCCMPoliciesAttack):
+        self.isSCCMPoliciesAttack = isSCCMPoliciesAttack
+    
+    def setSCCMPoliciesOptions(self, sccm_policies_clientname, sccm_policies_sleep):
+        self.SCCMPoliciesClientname = sccm_policies_clientname
+        self.SCCMPoliciesSleep = sccm_policies_sleep
+    
+    def setIsSCCMDPAttack(self, isSCCMDPAttack):
+        self.isSCCMDPAttack = isSCCMDPAttack
+    
+    def setSCCMDPOptions(self, sccm_dp_extensions, sccm_dp_files):
+        self.SCCMDPExtensions = sccm_dp_extensions
+        self.SCCMDPFiles = sccm_dp_files
 
     def setAltName(self, altName):
         self.altName = altName
